package com.eis.hw;

import java.io.Serializable;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.resource.Referenceable;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnectionFactory;

/**
 * Eine Instanz dieser Klasse wird im Rahmen der Server-Initialisierung
 * in den JNDI abgelegt und kann dort von Clients referenziert werden.
 * 
 * Diese Klasse ist serialisierbar, da sie an Client JVM verteilt werden muss
 * und sie ist referenceable um im JNDI zu funktionieren.
 * 
 */
final class HelloWorldConnectionFactoryImpl implements 
    HelloWorldConnectionFactory, Serializable, Referenceable {

	public final String test = "HelloWorldConnectionFactoryImpl";

	/**
	 * Die UID der Klasse. 
	 */
	private static final long serialVersionUID = 666L;

	final ConnectionManager cm;

	final ManagedConnectionFactory mcf;

	/**
	 * Dieser Konstruktor wird im Rahmen der Server-Initialisierung
	 * gerufen, die Instanz wird im JNDI abgelegt.
	 * 
	 * @param cm vom Server bereitgestellter ConnectionManager 
	 * @param mcf die ManagedConnectionFactory Implementierung,
	 * mit ihr wird der ConnectionManager ein Connection Handle
	 * bereitstellten. 
	 */
	HelloWorldConnectionFactoryImpl(final ConnectionManager cm,
			final ManagedConnectionFactory mcf) {
		this.cm = cm;
		this.mcf = mcf;
	}

	/**
	 * @see com.eis.hw.HelloWorldConnectionFactory#getConnection()
	 */
	public HelloWorldConnection getConnection() throws ResourceException {
        /*
         * Alternativ: Standard-ConnectionRequestInfo erzeugen
         */ 
		HelloWorldConnectionRequestInfo cri = null; 		
		return (HelloWorldConnection) getConnection(cri);
	}

	/**
	 * getConnection erzeugt ein Connection Handle indem es dessen
	 * delegiert die Implementierung an den ConnectionManager. 
	 * 
	 * ConnectionManager.allocateConnection 
	 * 
	 * - die ResourceException kommt von allocateConnection, ist aber nicht zwingend

	 * @see com.eis.hw.HelloWorldConnectionFactory#getConnection(javax.resource.spi.ConnectionRequestInfo)
	 */
	public HelloWorldConnection getConnection(final HelloWorldConnectionRequestInfo cri)
			throws ResourceException {
		return (HelloWorldConnection) this.cm.allocateConnection(this.mcf, cri);
	}

	// -- Standardimlplementierung für Referenceable -- //
	/**
	 * Stores the provided reference.
	 */
	private Reference reference = null;

	/**
	 * Set the reference.
	 */
	public final void setReference(final Reference reference) {
		this.reference = reference;
	}

	/**
	 * Get the stored reference.
	 */
	public final Reference getReference() throws NamingException {
		return this.reference;
	}
}