package com.eis.hw;

import javax.resource.ResourceException;
import javax.resource.spi.ManagedConnection;

/**
 * Implementierung des Connection Handles. 
 *
 * Methodenaufrufe werden typischer an die 
 * {@link ManagedConnectionImpl} Instanz delegiert, die mit diesem
 * ConnectionHandle assoziiert ist.
 *   
 * Clients, die ihre ConnectionHandle nicht schließen, werden
 * {@link ManagedConnection}s blockieren. Ein idle-Timeout wird
 * es nicht geben, da der Container nur Instanzen im Pool in dieser
 * Hinsicht überwacht. Solche Clients brechen aber signifikant den
 * Vertrag für diese Schnittstelle.
 */
final class HelloWorldConnectionImpl implements HelloWorldConnection {

	/**
	 * Ein Connection Handle darf nur mit genau einer Managed Connection
	 * assoziiert sein - deshalb ein Feld dafür
	 */
	private ManagedConnectionImpl managedConnection = null;

	/**
	 * Injiziert die mit diesem Connection Handle verknüpfte {@link ManagedConnection}.
	 * 
	 * Im Rahmen des Connection Sharing kann auch null übergeben werden, 
	 * das signalisiert, dass das ConnectionHandle vorübergehend nicht
	 * mit einer {@link ManagedConnection} assoziiert ist. Das ConnectionHandle
	 * muss aber intakt bleiben, da zu einem späteren Zeitpunkt die
	 * alte Assoziation wieder hergestellt wird. 
	 * 
	 * @param managedConnection
	 */
	void setManagedConnetion(ManagedConnectionImpl managedConnection) {
		this.managedConnection = managedConnection;
	}
	
	/**
	 * Nach Benutzung vom Client zu rufen, damit die ManagedConnection
	 * anderen Clients zur Verfügung stehen kann.
	 * 
	 * @see com.eis.hw.HelloWorldConnection#close()
	 */
	public void close() {
		if (this.managedConnection != null) {
		    this.managedConnection.closeHandle(this);
		}
	}

	/**
	 * @throws ResourceException 
	 * @see com.eis.hw.HelloWorldConnection#helloWorldCall()
	 */
	public String helloWorldCall() throws ResourceException {
		if (this.managedConnection != null) {
			return this.managedConnection.doHelloWorldCall();
		}
		throw new ResourceException("Missing ManagedConnection (Timeout?!)");
	}
}